*****************************************************************************
* Modulname:	D->LIB
* Modultype:	Library
* Dest.Comp.:	HP48, ROM rev A-J
* Language:	System RPL/Saturn assembler
* Author:	Detlef Mueller, e-mail detlef@dmhh.hanse.de
*		(c) 1993, all rights reserved
* Interface:	Uses the variables of the current active directory
* Description:	Creates a library from the variables of the current active
*		directory; the directory format is similar to the format
*		defined in USRLIB.DOC
* To do:
* Edition History :
*     1.610 18.01.1993	DM	unlinked from MKLIB 1.602 and reviewed
*
* $Id: D2LIB.S 1.1 1996/08/18 23:12:29 Detlef Exp $
*
* $Log: D2LIB.S $
* Revision 1.1  1996/08/18 23:12:29  Detlef
* Initial revision
*
*****************************************************************************


    TITLE	Library Creator


*****************************************************************************
*		ROM ID DEFINITIONS -> 1220
*****************************************************************************


xROMID		4C4

ASSEMBLE
RomID		EQU	#4C4
RPL


*****************************************************************************
*		EXTERNAL DEFINITIONS, LAYOUT
*****************************************************************************


EXTERNAL	xABOUT		( *1	Page 1* )

EXTERNAL	Error

EXTERNAL	xD\8DLIB	( *2* )

EXTERNAL	#>hxs3
EXTERNAL	ReplID
EXTERNAL	{}-
EXTERNAL	PosHxs
EXTERNAL	ob>hxs

* Assembler Jumptable


*****************************************************************************
*		GLOBAL DEFINES
*****************************************************************************


DEFINE	NULLHXS?	NULL$?		( *is legal* )


*****************************************************************************
*		OBJECTS
*****************************************************************************


*** cfg *********************************************************************
* Interface :	( --> )
* Description :	Warmstart o-matic attach to home directory
* Uses :	RomID
* Ed. Hist. :
*     1.000 21.11.1991 DM	start
*****************************************************************************

ASSEMBLE
=cfg
RPL
::
    [#] RomID TOSRRP			( *Autoattach* )
;

*** msg *********************************************************************
* Description : Message table and error calls
* Ed. History :
*     1.000 21.11.1991 DM	start
*****************************************************************************

ASSEMBLE
=msg
RPL
ARRY
[
    "$CONFIG Not a Program"
    "Missing $ROMID"
    "Found ID Name>16 Chars"
    "$ROMID Not Real/Binary"
    "$ROMID Out of Range"
    "$HIDDEN Not a List"
    "$VISIBLE Not a List"
    "$VARS Not a List"
    "$MESSAGE Not a List"
    "Found 0-ID"
*    1234567890123456789012
]

DEFINE	$CfgNotProg	:: ONE Error ;
DEFINE	Miss$ROMID	:: TWO Error ;
DEFINE	IDtoLong	:: THREE Error ;
DEFINE	$RidInvT	FOUR Error		( *! Not quoted !* )
DEFINE	$RidOutOR	:: FIVE Error ;
DEFINE	$HidNotList	:: SIX Error ;
DEFINE	$VisNotList	:: SEVEN Error ;
DEFINE	$VarNotList	:: EIGHT Error ;
DEFINE	$MesNotList	:: NINE Error ;
DEFINE	FndNullID	:: TEN Error ;

** xABOUT *******************************************************************
* Interface :	( --> )
* Description :	The nasty 'who am I' message
* Ed. Hist. :
*     1.001 18.01.1993 DM	alpha
*****************************************************************************

ASSEMBLE
	CON(1)	8		* Tell parser 'Non algebraic'
RPL
xNAME	ABOUT
::
    CK0

    "detlef@dmhh.hanse.de"
    "e-mail:"
    "Detlef M\DCller"
    "\A9 1993 by"
    "D\8DLIB Version 1.0"

    EIGHT THREE
    DO
	TWENTYTHREE OVERLEN$ #- #2/
	Blank$ !insert$
	INDEX@ DISPN
    LOOP

    FIFTEEN 400 TWENTYFOUR 300
    setbeep setbeep
    SetDA12Temp
;

** Error ********************************************************************
* Interface :	( #[MsgNr] --> )
* Description :	Error exit point
* Uses :	RomID
* Ed. Hist. :
*     1.000 26.08.1991 DM	alpha
*****************************************************************************

NULLNAME	Error
::
    [#]
    ASSEMBLE
	CON(5)	(RomID)*256
    RPL
    #+ ERROROUT
;

** xD->LIB ******************************************************************
* Interface :	( --> lib )
* Description :	Create a library from the variables of the current directory
* Uses :	vars of the curr.dir.
* Calls :	ob>hxs {}- PosHxs ReplID
*		=Loop Pop# SavPtr =PUSHhxsLoop #>hxs =DoCRC =GETPTRLOOP
* Defines :	#>hxs3
*		#>hxs
* Ed. Hist. :
*     1.000 21.11.1991 DM	alpha
*	100 18.01.1993 DM	modified for running standalone
*****************************************************************************

DEFINE	ROMID		ID $ROMID
DEFINE	TITLE		ID $TITLE
DEFINE	MESSAGE		ID $MESSAGE
DEFINE	VARS		ID $VARS
DEFINE	VISIBLE		ID $VISIBLE
DEFINE	HIDDEN		ID $HIDDEN
DEFINE	CONFIG		ID $CONFIG

DEFINE	@tmpa		1GETLAM		( *temp A* )
DEFINE	!tmpa		1PUTLAM
DEFINE	@tmpb		2GETLAM		( *temp B* )
DEFINE	!tmpb		2PUTLAM
DEFINE	@hidden		3GETLAM		( *{id} - unnamed obs* )
DEFINE	!hidden		3PUTLAM
DEFINE	@msgtab		4GETLAM		( *hxs[arry[$]] - message table* )
DEFINE	!msgtab		4PUTLAM
DEFINE	@hash		5GETLAM		( *hxs[hxs]] - hash table* )
DEFINE	!hash		5PUTLAM
DEFINE	@cfg		6GETLAM		( *hxs[::] - config code* )
DEFINE	!cfg		6PUTLAM
DEFINE	@lib		7GETLAM		( *hxs - library and tmp* )
DEFINE	!lib		7PUTLAM
DEFINE	@incl		8GETLAM		( *{id} - obs to add to library* )
DEFINE	!incl		8PUTLAM
DEFINE	@romid		9GETLAM		( *hxs[3] - target romid* )
DEFINE	!romid		9PUTLAM

DEFINE	D->Llams	NINE		( *Need a nine-lam temp env.* )


ASSEMBLE
	CON(1)	8		* Tell parser 'Non algebraic'
RPL
xNAME	D\8DLIB
::
    CK0
				( *ANALYZING PART* )

				( *Get and check $ROMID* )
    ' ROMID SAFE@_HERE
    ?SKIP Miss$ROMID			( *unable to find RID var* )

    ::  DUPTYPEREAL? case COERCE	( % --> # )
	DUPTYPEHSTR? case HXS>#		( hxs --> # )
	$RidInvT			( *RID have invallid type* )
    ;

    # 7FF OVER#< case $RidOutOR		( *RID out of range* )

    BlankDA2

    "Analyzing\1F" DISPROW4

    NULLNAME #>hxs3			( # --> hxs[3] )
    CODE
	P=	2	want to push HXS 3
#>hxs	GOSUBL	Pop#
	GOSUBL	SavPtr
	GOVLNG	=PUSHhxsLoop
    ENDCODE
				( *Make TEMP env.* )
					( *Initial: romid,* )
    DOVARS ?ATTNQUIT			( *incl,* )
    NULLHXS DUP DUPDUP			( *lib, cfg, hash, msgtab,* )
    NULL{} ZERO				( *hidden, tmpb,* )
    {	ROMID TITLE
	MESSAGE VARS
	VISIBLE HIDDEN CONFIG
    }					( *tmpa* )


    ' NULLLAM D->Llams NDUPN {}N BIND

				( *Get and check $MESSAGE* )
    ' MESSAGE SAFE@_HERE
    IT
    ::	DUPTYPELIST? ?SKIP $MesNotList	( *Message table not a list* )
	DUPNULL{}? caseDROP		( *MsgTab nulllist, ignore* )

	INNERCOMP			( --> ..obn #n )
	HXS A C2A2010000		( *DOCSTR, dim = 1..* )
	OVER #>HXS &$SWAP		( *..elem. cnt. = n* )
					( --> ..obn hxs #n )
	ZERO_DO (DO)
	    ISTOP-INDEX #1+ROLL
	    DO>STR ob>hxs
	    SIX OVER LENHXS SUBHXS	( *Strip DOCSTR* )
	    !!append$
	LOOP

	[#] DOARRY CHANGETYPE

	ob>hxs !msgtab
    ;

				( *Get and check $CONFIG* )
    ' CONFIG SAFE@_HERE
    IT
    ::	DUPTYPECOL? ?SKIP $CfgNotProg	( *Not a program* )

	ob>hxs !cfg
    ;

				( *Get and check $VARS* )
    ' VARS SAFE@_HERE
    IT
    ::	DUPTYPELIST? ?SKIP $VarNotList	( *Not a list* )

	@tmpa OVER {}- &COMP !tmpa	( $VARS + [ctrl - $VARS] )
    ;

    @incl @tmpa {}- !incl		( *correct incl* )

				( *Get and check $HIDDEN & $VISIBLE* )
    ::
	' HIDDEN SAFE@_HERE
	case				( *$HIDDEN exists..* )
	::  DUPTYPELIST?
	    ?SKIP $HidNotList		( *..but not as list* )

	    DUP @incl {}- {}- !hidden	( hid - [hid - inc] )
	;

	' VISIBLE SAFE@_HERE NOT?SEMI	( *$VISIBLE exists..* )
	DUPTYPELIST?
	?SKIP $VisNotList		( *..but not as list* )

	@incl SWAP {}- !hidden		( inc - vis )
    ;

				( *HASH TABLE CREATION* )

    @incl @hidden {}-
    DUPLENCOMP DUP !tmpa #0=
    ITE_DROP				( *No visible vars* )
    ::	?ATTNQUIT

	"Creating Hash Table\1F" DISPROW4

	DUP @hidden &COMP !incl		( *order incl - vis 1st, rest hid* )
	@tmpa				( --> {vis} #vis )

	ZERO_DO (DO)
	    DUPINDEX@ #1+ NTHCOMPDROP	( --> .. {} id )
	    ob>hxs
	    SIX OVER LENHXS SUBHXS	( *Strip DOIDNT* )
	    INDEX@ #>hxs3 &$SWAP	( *Append rp #* )
	LOOP

	DROP @tmpa {}N !tmpb

	NULLHXS SIXTEEN NDUPN DROP	( --> id16 .. id1 )
	@tmpa
	#1+_ONE_DO (DO)
	    @tmpb INDEX@ NTHCOMPDROP	( --> ids id )
	    DUP LENHXS #5- #2/		( --> ids hxs[id] #lenid )

	    DUP#0=case FndNullID	( *should never happen* )
	    SIXTEEN OVER#< case IDtoLong

	    DUP#2+			( --> ids hxs[id] # )
	    ROLL ROT &$SWAP		( --> ids idn' # )
	    UNROLL
	LOOP

	?ATTNQUIT

	NULLHXS				( --> id16 .. id1 hxs )
	SIXTEEN
	ZERO_DO (DO)
	    OVER NULLHXS?
	    ITE
	    ::  SWAPDROP HXS 5 0	( --> ids hxs[n] hxs[0] )
	    ;
	    ::  SEVENTEEN INDEX@#-	( --> ids hxs[n] #offs )
		FIVE #*
		OVER LENHXS #+ #>HXS
		UNROTSWAP		( --> ids hxs[offs] hxs[n] id )
		&$SWAP
	    ;

	    @hash SWAP&$ !hash
	LOOP
					( --> hxs[names] )
	@hash OVER LENHXS #5+		( --> hxs[n] hash len_names )
	#>HXS &$
	OVER &$ !hash

	NULLHXS				( --> hxs[n] hxs )
	@tmpa
	#1+_ONE_DO (DO)
	    @tmpb INDEX@ NTHCOMPDROP	( --> hxs[n] hxs hxs[id] )
	    3PICKSWAP			( --> hxs[n] hxs hxs[n] hxs[id] )
	    ONE PosHxs			( --> hxs[n] hxs #pos )
	    3PICK LENHXS SWAP#-
	    OVER LENHXS #+ #1+ #>HXS &$
	LOOP

	@hash
	UNROTSWAPDROP &$ ob>hxs !hash
    ;

				( *LINK TABLE CREATION AND OBJECTS ADDING* )

    ?ATTNQUIT

    @incl DUPNULL{}?
    ITE
    ::	DROP NULLHXS			( *no obs to include* )
    ;
    ::	"Creating Link Table\1F" DISPROW4

	LENCOMP DUP !tmpa

	#1+_ONE_DO (DO)
	    @incl INDEX@ NTHCOMPDROP
	    ob>hxs			( --> .. hxs[id] )
	LOOP

	@tmpa {}N !tmpb
	@tmpa

	@cfg ReplID !cfg		( *ReplID modifies tmpa !* )

	NULLHXS SWAP			( --> hxs[lt] #cnt )
	#1+_ONE_DO (DO)
	    @incl INDEX@ NTHCOMPDROP

	    "Adding " OVER ID>$ &$ DISPROW5
	    @hidden OVER EQUALPOSCOMP
	    UNROT			( --> t/f hxs[lt] id )
	    SAFE@_HERE DROP ob>hxs	( --> t/f hxs[lt] hxs[ob] )
	    ReplID			( --> t/f hxs[lt] hxs[ob]' )
	    @lib ZERO !lib
	    SWAP !insert$		( --> t/f hxs[lt] hxs[ob+lib] )
	    OVER LENHXS #10+
	    OVER LENHXS #+		( --> t/f hxs[lt] hxs[lib] #offs )
	    ZERO SWAP#- #>HXS
	    ROTSWAP &$SWAP		( --> t/f hxs[lt] hxs[lib] )

	    ROT #0=			( *Not hidden,* )
	    OVER HXS># [#] DOCOL #= AND	( *and is program* )
	    IT
	    ::
		   ( * x<<  xALG->    x<<       DOLAM * )
		HXS 1E E1632BEF22D6E20E16321C432D6E20
		   ( *           DOLAM     xRPN->     * )

		OVER SIX FIFTEEN SUBHXS
		ONE PosHxs		( --> hxs[lt] hxs[lib] # )
		#0=ITE
		    HXS 1 8
		    HXS 3 0

		@romid &$ INDEX@ #1- #>hxs3 &$
		!!insert$
	    ;

	    !lib			( --> hxs[lt] )

	    ?ATTNQUIT
	LOOP

	ZERO !tmpb ob>hxs
	NULL$ DISPROW5
    ;

				( *LIBRARY CREATION * )

    "Creating Library\1F" DISPROW4
    !tmpa				( *Link table to tmpa* )

    @lib ZERO !lib			( --> hxs[obs] )
    @cfg !insert$			( --> hxs[cfg_obs] )
    @msgtab !!insert$			( --> hxs[msg_cfg_obs] )
    !lib				( *Library object part to lib* )

				( *Create library header* )
    ' TITLE SAFE@_HERE
    ITE					( *Title exists* )
    ::	DO>STR DUPLEN$			( *enshure string* )

    CODE
	P=	1
	GOLONG	#>hxs
    ENDCODE
					( --> $ hxs[len$] )
	SWAPOVER HXS 2 0 EQUALcase DROP	( *$ out of range -> no title* )
	ob>hxs				( --> hxs[len] hxs[$] )
	ELEVEN OVER LENHXS SUBHXS	( *strip DOCSTR, len* )
	OVER &$ &$			( --> hxs[len $ len] )
    ;
	HXS 2 0				( *No title* )

    @romid &$				( --> hxs[tit_rid] )

    @hash NULLHXS?
    ITE
	ZERO				( *No hash table* )
    ::	@lib LENHXS @tmpa LENHXS #+ TWENTY #+
    ;
    #>HXS				( *Ptr to hash table* )
    &$					( --> hxs[tit_rid_hash] )

    @msgtab NULLHXS?
    ITE
	ZERO
	FIFTEEN
    #>HXS				( *Ptr to message table* )
    &$					( --> hxs[tit_rid_hash_msg] )

    @tmpa NULLHXS?
    ITE
	ZERO
    ::	@lib LENHXS #10+
    ;
    #>HXS				( *Ptr to link table* )
    &$					( --> hxs[tit_rid_hash_msg_link] )

    @cfg NULLHXS?
    ITE
	ZERO
    ::	@msgtab LENHXS #5+
    ;
    #>HXS				( *Ptr to configuration code* )
    &$					( --> hxs[tit_rid_hash_msg_link_cfg] )

    @lib @tmpa @hash			( --> header obs link hash )

    ABND

    HXS 4 0 !append$ !append$ !append$
    SWAP !insert$			( --> hxs-lib )

    CODE			( *hxs becomes lib, calc and patch CRC* )
* CPU			A	C	D0	D1
	GOSUB	SavPtr

	C=DAT1	A		->hxs
	D0=C				->hxs
	LA(5)	=DOLIB	lib_prol
	DAT0=A	A	    *** Change prologue

	D0=D0+	5			->len

	A=DAT0	A	len
	A=A-CON	A,4	len - CRClen

	GOSBVL	=DoCRC	CRC	xx	->CRC	xx

	DAT0=A	4	    *** Patch CRC

	GOVLNG	=GETPTRLOOP
    ENDCODE
;

** ReplID *******************************************************************
* Interface :	( hxs[ob] --> hxs[ob]' )
* Description :	Change known IDs to ROMPTR
* Uses:		tmpa tmpb romid
* Modifies :	tmpa
* Calls :	PosHxs
* Ed. Hist. :
*     1.000 21.11.1991 DM	alpha
*****************************************************************************

NULLNAME ReplID
::
    HXS 5 84E20				( --> hxs[o] hxs[DOIDNT] )
    SWAP				( --> hxs[f] hxs[ob] )
    2DUPSWAP ONE PosHxs			( --> f s #p )

    DUP#0=csedrp SWAPDROP		( *no ID found* )

    BEGIN
	2DUP 2DUP #5+ DUP#1+ SUBHXS
	HXS># #2* SEVEN #+DUP !tmpa	( --> f s #p s #p #idlen )
	OVER#+ #1- SUBHXS		( --> f s #p hxs[id] )
	@tmpb SWAP EQUALPOSCOMP
	DUP#0=ITE
	::  DROP
	    #5+				( --> f s #p' )
	;
	::  HXS 5 29E20			( *DOROMP* )
	    @romid &$ SWAP#1- #>hxs3
	    !!append$			( --> f s #p hxs[rp] )
	    UNROT			( --> f rp s #p )
	    2DUP #1-1SWAP SUBHXS	( --> f rp >s< #p > )
	    4ROLL !!append$		( --> f >s< #p >r )
	    ROT 3PICK			( --> f #p >r >s< #p )
	    @tmpa #+OVER LENHXS		( --> f #p >r >s< #p+lf #len )
	    SUBHXS !!append$		( --> f #p >r< )
	    SWAP ELEVEN #+		( --> f s #p' )
	;

	OVER 4PICK ROT			( --> f s s f #p )
	PosHxs DUP#0=			( --> f s #p' t/f )
    UNTIL

    DROPSWAPDROP
;

** {}- **********************************************************************
* Interface :	( {1} {2} --> {1-2} )
* Description :	LISP diff
* Ed. Hist. :
*     1.000 21.11.1991 DM	alpha
*****************************************************************************

NULLNAME	{}-
::
    DUPNULL{}? caseDROP
    SWAP				( --> {2} {1} )
    DUPNULL{}? case SWAPDROP

    INNERDUP #2+ROLL ZERO ROT		( --> on .. ox .. o1 {2} #0 #n )
    ZERO_DO (DO)
	DUP #3+PICK			( --> on .. ox .. o1 {2} #x ox )
	3PICKSWAP EQUALPOSCOMP		( --> on .. ox .. o1 {2} #x t/f )
	#0=ITE
	    #1+
	::  DUP #3+ ROLL DROP
	;
    LOOP

    SWAPDROP {}N
;

** PosHxs *******************************************************************
* Interface :	( hxs(1) hxs(2) # --> #0/#pos )
* Description :	Starting at offset #, search the 1st occurence of hxs(2)
*		in hxs(1). Return #0 on no-match.
* Calls :	Pop# SavPtr =PUSH#ALOOP
* Defines :
* Ed. Hist. :
*     1.000 12.12.1991 DM	alpha
*****************************************************************************

NULLNAME	PosHxs
CODE
* CPU			A	B	C	D	D0	D1	Rx
	GOSUB	Pop#	pos

	C=DAT1	A			->hxs[f]
	R1=C	A							1:->hxs[f]
	D1=D1+	5
	D=D+1	A	    *** Pop hxs[f]

	C=DAT1	A			->hxs[s]
	R3=C	A							3:->hxs[s]
	D1=D1+	5
	D=D+1	A	    *** Pop hxs[s]

	GOSUB	SavPtr

	?A=0	A	    *** Start = 0 ?
	GOYES	Match	    *** yep, ret. 0
	A=A-1	A	--pos

	C=R3	A			->hxs[s]
	D0=C						->hxs[s]
	D0=D0+	5					->ls
	C=DAT0	A			ls
	C=C-CON	A,5
	?C=0	A	    *** len[s] = 0 ?
	GOYES	NoMatch	    *** yep, ret. 0
	B=C	A		ls
	D0=D0+	5					->s
	CD0EX				->s
	R3=C	A							3:->s
	C=C+A	A			->s'
	D0=C						->s'

	C=R1	A			->hxs[f]
	D1=C							->hxs[f]
	D1=D1+	5						->lf
	C=DAT1	A			lf
	C=C-CON	A,5
	?C=0	A	    *** len[fnd] = 0 ?
	GOYES	NoMatch	    *** yep, ret. 0

	D1=D1+	5						->f
	CD1EX				->f			lf
	R1=C	A							1:->f
	CD1EX				lf			->f

	?C>B	A	    *** lf > ls ?
	GOYES	NoMatch	    *** yep, ret. 0

	C=C-1	A		lf-1
	R2=C	A							2:lf-1
	B=B-C	A		ls-lf+1
	?A>=B	A	    *** pos >= len[s] ?
	GOYES	NoMatch	    *** yep, ret. 0

	B=B-A	A		ls-lf-pos

	A=0	B	0		0
	C=0	B

* CPU			x	trys	x	x	&s	&f
*			R0:x	R1:&f	R2:lf-1	R3:&s

	GONC	SEntry	    *** BET

NoMatch	A=0	A
Match	GOVLNG	=PUSH#ALOOP

SLoop	A=DAT0	1	[s]
	C=DAT1	1			[f]
	D0=D0+	1					++s
	?A=C	B	    *** first char equal ?
	GOYES	Match?	    *** yep, check for match
SEntry	B=B-1	A		--trys
	GONC	SLoop
	GOC	NoMatch	    *** BET

Match?	CD0EX				->s
	D0=C						->s
	R0=C.F	A							0:->s
	C=R2.F	A			lf-1
	D=C	A				len
	C=0	B			0
	GOC	MEntry	    *** BET

MLoop	A=DAT0	1	[s]
	C=DAT1	1			[f]
	?A=C	B	    *** equal ?
	GOYES	MNxt	    *** yep, check next

	C=R0.F	A			->s
	D0=C						->s
	C=R1.F	A			->f
	D1=C							->f
	C=0	B			0
	GONC	SEntry	    *** BET

MNxt	D0=D0+	1					->s'
MEntry	D1=D1+	1						->f'
	D=D-1	A				--len
	GONC	MLoop

	A=R0	A	->ns
	C=R3	A			->s
	A=A-C	A	pos
	GONC	Match	    *** BET
ENDCODE

** ob>hxs *******************************************************************
* Interface :	( ob --> hxs )
* Description :	Move ob into body of a hxs
* Calls :	SavPtr =SKIPOB =MAKE$N MOVEDOWN =GPOverWrR0Lp
* Defines :
* Ed. Hist. :
*     1.000 13.02.1992 DM	alpha
*	100 18.04.1992 DM	rewritten
*****************************************************************************

NULLNAME	ob>hxs
CODE
* CPU			A	C	D0	D1	R0	R1
	GOSUB	SavPtr

	A=DAT1	A	->ob
	R1=A							->ob
	D0=A
	GOSBVL	=SKIPOB	xx	xx	->past ob
	CD0EX			->past ob
	A=R1		->ob
	C=C-A	A			len ob
	GOSBVL	=MAKE$N	len val	->stk	->body	->stk	->$
	D0=D0-	10			->$
	LC(5)	=DOHSTR		hxs
	DAT0=C	A	    We want HXS !
	D0=D0+	10			->body
	AD0EX		->body		len val
	D1=A					->body
	D0=D0-	5			len
	CD0EX			len
	A=R1		->ob
	D0=A				->ob
	GOSBVL	=MOVEDOWN
	GOVLNG	=GPOverWrR0Lp
ENDCODE

*****************************************************************************
*		Assembler Jumptable
*****************************************************************************

ASSEMBLE

Pop#	GOVLNG	=POP#
SavPtr	GOVLNG	=SAVPTR

RPL

*****************************************************************************
*			End of D2LIB.S
*****************************************************************************
